@@ -1,5 +1,6 @@
-""" `tldextract` accurately separates a URL's subdomain, domain, and public suffix,
-using the Public Suffix List (PSL).
+"""`tldextract` accurately separates a URL's subdomain, domain, and public suffix.
+
+It does this via the Public Suffix List (PSL).
 
     >>> import tldextract
 
@@ -48,11 +49,22 @@ or suffix were found:
     '127.0.0.1'
 """
 
+from __future__ import annotations
+
 import logging
 import os
 import urllib.parse
 from functools import wraps
-from typing import FrozenSet, List, NamedTuple, Optional, Sequence, Union
+from typing import (
+    Collection,
+    Dict,
+    FrozenSet,
+    List,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Union,
+)
 
 import idna
 
@@ -88,8 +100,8 @@ class ExtractResult(NamedTuple):
         >>> extract('http://localhost:8080').registered_domain
         ''
         """
-        if self.domain and self.suffix:
-            return self.domain + "." + self.suffix
+        if self.suffix and self.domain:
+            return f"{self.domain}.{self.suffix}"
         return ""
 
     @property
@@ -102,7 +114,7 @@ class ExtractResult(NamedTuple):
         >>> extract('http://localhost:8080').fqdn
         ''
         """
-        if self.domain and self.suffix:
+        if self.suffix and self.domain:
             # Disable bogus lint error (https://github.com/PyCQA/pylint/issues/2568)
             # pylint: disable-next=not-an-iterable
             return ".".join(i for i in self if i)
@@ -111,7 +123,7 @@ class ExtractResult(NamedTuple):
     @property
     def ipv4(self) -> str:
         """
-        Returns the ipv4 if that is what the presented domain/url is
+        Returns the ipv4 if that is what the presented domain/url is.
 
         >>> extract('http://127.0.0.1/path/to/file').ipv4
         '127.0.0.1'
@@ -126,8 +138,7 @@ class ExtractResult(NamedTuple):
 
 
 class TLDExtract:
-    """A callable for extracting, subdomain, domain, and suffix components from
-    a URL."""
+    """A callable for extracting, subdomain, domain, and suffix components from a URL."""
 
     # TODO: Agreed with Pylint: too-many-arguments
     def __init__(  # pylint: disable=too-many-arguments
@@ -139,9 +150,7 @@ class TLDExtract:
         extra_suffixes: Sequence[str] = (),
         cache_fetch_timeout: Union[str, float, None] = CACHE_TIMEOUT,
     ) -> None:
-        """
-        Constructs a callable for extracting subdomain, domain, and suffix
-        components from a URL.
+        """Construct a callable for extracting subdomain, domain, and suffix components from a URL.
 
         Upon calling it, it first checks for a JSON in `cache_dir`. By default,
         the `cache_dir` will live in the tldextract directory. You can disable
@@ -204,17 +213,17 @@ class TLDExtract:
         self._cache = DiskCache(cache_dir)
 
     def __call__(
-        self, url: str, include_psl_private_domains: Optional[bool] = None
+        self, url: str, include_psl_private_domains: bool | None = None
     ) -> ExtractResult:
         """Alias for `extract_str`."""
         return self.extract_str(url, include_psl_private_domains)
 
     def extract_str(
-        self, url: str, include_psl_private_domains: Optional[bool] = None
+        self, url: str, include_psl_private_domains: bool | None = None
     ) -> ExtractResult:
-        """
-        Takes a string URL and splits it into its subdomain, domain, and
-        suffix (effective TLD, gTLD, ccTLD, etc.) components.
+        """Take a string URL and splits it into its subdomain, domain, and suffix components.
+
+        I.e. its effective TLD, gTLD, ccTLD, etc. components.
 
         >>> extractor = TLDExtract()
         >>> extractor.extract_str('http://forums.news.cnn.com/')
@@ -229,10 +238,10 @@ class TLDExtract:
         url: Union[urllib.parse.ParseResult, urllib.parse.SplitResult],
         include_psl_private_domains: Optional[bool] = None,
     ) -> ExtractResult:
-        """
-        Takes the output of urllib.parse URL parsing methods and further splits
-        the parsed URL into its subdomain, domain, and suffix (effective TLD,
-        gTLD, ccTLD, etc.) components.
+        """Take the output of urllib.parse URL parsing methods and further splits the parsed URL.
+
+        Splits the parsed URL into its subdomain, domain, and suffix
+        components, i.e. its effective TLD, gTLD, ccTLD, etc. components.
 
         This method is like `extract_str` but faster, as the string's domain
         name has already been parsed.
@@ -259,11 +268,11 @@ class TLDExtract:
             labels, include_psl_private_domains=include_psl_private_domains
         )
 
-        suffix = ".".join(labels[suffix_index:])
-        if not suffix and netloc and looks_like_ip(netloc):
+        if suffix_index == len(labels) and netloc and looks_like_ip(netloc):
             return ExtractResult("", netloc, "")
 
-        subdomain = ".".join(labels[: suffix_index - 1]) if suffix_index else ""
+        suffix = ".".join(labels[suffix_index:]) if suffix_index != len(labels) else ""
+        subdomain = ".".join(labels[: suffix_index - 1]) if suffix_index >= 2 else ""
         domain = labels[suffix_index - 1] if suffix_index else ""
         return ExtractResult(subdomain, domain, suffix)
 
@@ -277,22 +286,23 @@ class TLDExtract:
     @property
     def tlds(self) -> List[str]:
         """
-        Returns the list of tld's used by default
+        Returns the list of tld's used by default.
 
         This will vary based on `include_psl_private_domains` and `extra_suffixes`
         """
         return list(self._get_tld_extractor().tlds())
 
-    def _get_tld_extractor(self) -> "_PublicSuffixListTLDExtractor":
-        """Get or compute this object's TLDExtractor. Looks up the TLDExtractor
-        in roughly the following order, based on the settings passed to
-        __init__:
+    def _get_tld_extractor(self) -> _PublicSuffixListTLDExtractor:
+        """Get or compute this object's TLDExtractor.
+
+        Looks up the TLDExtractor in roughly the following order, based on the
+        settings passed to __init__:
 
         1. Memoized on `self`
         2. Local system _cache file
         3. Remote PSL, over HTTP
-        4. Bundled PSL snapshot file"""
-
+        4. Bundled PSL snapshot file
+        """
         if self._extractor:
             return self._extractor
 
@@ -318,6 +328,37 @@ class TLDExtract:
 TLD_EXTRACTOR = TLDExtract()
 
 
+class Trie:
+    """Trie for storing eTLDs with their labels in reverse-order."""
+
+    def __init__(self, matches: Optional[Dict] = None, end: bool = False) -> None:
+        self.matches = matches if matches else {}
+        self.end = end
+
+    @staticmethod
+    def create(suffixes: Collection[str]) -> Trie:
+        """Create a Trie from a list of suffixes and return its root node."""
+        root_node = Trie()
+
+        for suffix in suffixes:
+            suffix_labels = suffix.split(".")
+            suffix_labels.reverse()
+            root_node.add_suffix(suffix_labels)
+
+        return root_node
+
+    def add_suffix(self, labels: List[str]) -> None:
+        """Append a suffix's labels to this Trie node."""
+        node = self
+
+        for label in labels:
+            if label not in node.matches:
+                node.matches[label] = Trie()
+            node = node.matches[label]
+
+        node.end = True
+
+
 @wraps(TLD_EXTRACTOR.__call__)
 def extract(  # pylint: disable=missing-function-docstring
     url: str, include_psl_private_domains: Optional[bool] = False
@@ -332,9 +373,7 @@ def update(*args, **kwargs):  # type: ignore[no-untyped-def]
 
 
 class _PublicSuffixListTLDExtractor:
-    """Wrapper around this project's main algo for PSL
-    lookups.
-    """
+    """Wrapper around this project's main algo for PSL lookups."""
 
     def __init__(
         self,
@@ -349,28 +388,8 @@ class _PublicSuffixListTLDExtractor:
         self.private_tlds = private_tlds
         self.tlds_incl_private = frozenset(public_tlds + private_tlds + extra_tlds)
         self.tlds_excl_private = frozenset(public_tlds + extra_tlds)
-
-        public_false_tlds = self.get_false_intermediate_suffixes(public_tlds)
-        private_false_tlds = self.get_false_intermediate_suffixes(private_tlds)
-        extra_false_tlds = self.get_false_intermediate_suffixes(extra_tlds)
-        self.false_tlds_incl_private = frozenset(
-            public_false_tlds + private_false_tlds + extra_false_tlds
-        )
-        self.false_tlds_excl_private = frozenset(public_false_tlds + extra_false_tlds)
-
-    def get_false_intermediate_suffixes(self, tlds: List[str]) -> List[str]:
-        """From list of suffixes, identify false intermediate suffixes.
-
-        Example: If valid TLDs include only ["a.b.c.d", "d"], then
-        ["b.c.d", "c.d"] are false intermediate suffixes.
-        """
-        valid_tlds = set(tlds)
-        false_tlds = set()
-        for tld in valid_tlds:
-            labels = tld.split(".")
-            variants = {".".join(labels[-i:]) for i in range(1, len(labels))}
-            false_tlds.update(variants)
-        return list(false_tlds.difference(valid_tlds))
+        self.tlds_incl_private_trie = Trie.create(self.tlds_incl_private)
+        self.tlds_excl_private_trie = Trie.create(self.tlds_excl_private)
 
     def tlds(
         self, include_psl_private_domains: Optional[bool] = None
@@ -385,53 +404,38 @@ class _PublicSuffixListTLDExtractor:
             else self.tlds_excl_private
         )
 
-    def false_tlds(
-        self, include_psl_private_domains: Optional[bool] = None
-    ) -> FrozenSet[str]:
-        """Get the currently filtered list of false intermediate suffixes."""
-        if include_psl_private_domains is None:
-            include_psl_private_domains = self.include_psl_private_domains
-
-        return (
-            self.false_tlds_incl_private
-            if include_psl_private_domains
-            else self.false_tlds_excl_private
-        )
-
     def suffix_index(
         self, spl: List[str], include_psl_private_domains: Optional[bool] = None
     ) -> int:
-        """Returns the index of the first suffix label.
-        Returns len(spl) if no suffix is found
+        """Return the index of the first suffix label.
+
+        Returns len(spl) if no suffix is found.
         """
-        tlds = self.tlds(include_psl_private_domains)
-        false_tlds = self.false_tlds(include_psl_private_domains)
+        node = (
+            self.tlds_incl_private_trie
+            if include_psl_private_domains
+            else self.tlds_excl_private_trie
+        )
         i = len(spl)
         j = i
-        maybe_tld = ""
-        prev_maybe_tld = ""
         for label in reversed(spl):
-            maybe_tld = (
-                f"{_decode_punycode(label)}.{maybe_tld}"
-                if maybe_tld
-                else _decode_punycode(label)
-            )
+            decoded_label = _decode_punycode(label)
+            if decoded_label in node.matches:
+                j -= 1
+                if node.matches[decoded_label].end:
+                    i = j
+                node = node.matches[decoded_label]
+                continue
 
-            if "!" + maybe_tld in tlds:
-                return j
-            if "*." + prev_maybe_tld in tlds:
+            is_wildcard = "*" in node.matches
+            if is_wildcard:
+                is_wildcard_exception = "!" + decoded_label in node.matches
+                if is_wildcard_exception:
+                    return j
                 return j - 1
 
-            if maybe_tld in tlds:
-                j -= 1
-                i = j
-                prev_maybe_tld = maybe_tld
-                continue
-            if maybe_tld in false_tlds:
-                j -= 1
-                prev_maybe_tld = maybe_tld
-                continue
             break
+
         return i
 
 
